package function.builtin.special;

import static function.builtin.EVAL.eval;

import function.*;
import sexpression.*;
import table.*;

public class SETF extends LispFunction {

    private ArgumentValidator argumentValidator;
    private ExecutionContext executionContext;

    public SETF() {
        this.argumentValidator = new ArgumentValidator("SETF");
        this.argumentValidator.setExactNumberOfArguments(2);
        this.argumentValidator.setFirstArgumentExpectedType(Symbol.class);
        this.executionContext = ExecutionContext.getInstance();
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        Cons rest = (Cons) argumentList.getRest();
        SExpression symbol = argumentList.getFirst();
        SExpression value = eval(rest.getFirst());

        SymbolTable table = findScopeOfSymbol(symbol);
        table.put(symbol.toString(), value);

        return value;
    }

    private SymbolTable findScopeOfSymbol(SExpression symbol) {
        SymbolTable table = executionContext.getScope();

        while (!isSymbolInTable(symbol, table) && !isGlobalTable(table))
            table = table.getParent();

        return table;
    }

    private boolean isSymbolInTable(SExpression symbol, SymbolTable table) {
        return table.contains(symbol.toString());
    }

    private boolean isGlobalTable(SymbolTable table) {
        return table.getParent() == null;
    }

    public boolean evaluateArguments() {
        return false;
    }

}
